Package: rfriend
Type: Package
Title: Provides Batch Functions and Visualisation for Basic Statistical
        Procedures
Version: 3.1.0
Authors@R: c(person("Sander H.", "van Delden", role = c("aut", "cre"), email = "plantmind@proton.me"))
Description: Designed to streamline data analysis and statistical testing, reducing the length of R 
    scripts while generating well-formatted outputs in 'pdf', 'Microsoft Word', and 'Microsoft Excel' 
    formats. In essence, the package contains functions which are sophisticated wrappers around 
    existing R functions that are called by using 'f_' (user f_riendly) prefix followed by the normal 
    function name. This third version of the 'rfriend' package focuses  primarily on data exploration, 
    including tools for creating summary tables, f_summary(), summary figures, f_scan(), 
    outlier detection and removal, f_outlier() and f_remove_outliers(), performing data transformations,
    f_boxcox() in part based on 'MASS/boxcox' and 'rcompanion', and f_bestNormalize() 
    which wraps and extends functionality from the 'bestNormalize' package. Furthermore, 'rfriend' 
    can automatically (or on request) generate visualizations such as boxplots, f_boxplot(), 
    QQ-plots, f_qqnorm(), histograms f_hist(), and density plots f_density(). Additionally, the package 
    includes several statistical test functions: f_aov(), f_chisq_test(), f_corplot(), f_kruskal_test(), 
    f_lmer(), f_glm(), f_t_test(), f_wilcox_test(),  for sequential testing and visualisation of the 
    similar named 'stats' functions. These functions, except for f_chisq_test(), support testing multiple
    response variables and predictors, while also handling assumption checks, data transformations, 
    and post hoc tests. Post hoc results are automatically summarized in a table using the compact letter 
    display (cld) format for easy interpretation. The package also provides a function to do model 
    comparison, f_model_comparison(), and several utility functions to simplify common R tasks. 
    For example, f_clear() clears the workspace and restarts R with a single command; f_setwd() sets 
    the working directory to match the directory of the current script; f_theme() quickly changes 
    'RStudio' themes; and f_factors() converts multiple columns of a data frame to factors, and much more. 
    If you encounter any issues or have feature requests, please feel free to contact me via email.
Note: When loading, both MuMIn and rstatix are imported. Since rstatix
        internally depends on broom, this may trigger a warning about
        S3 method overwrites, specifically for nobs.fitdistr and
        nobs.multinom. These warnings are harmless and do not affect
        functionality.
License: GPL-3
Encoding: UTF-8
Depends: R (>= 4.4.0)
Imports: bestNormalize, crayon, DHARMa, dplyr, emmeans, ggplot2,
        grDevices, gridExtra, knitr, lme4, lmerTest, magick, multcomp,
        multcompView, MuMIn, nortest, pander, png, rlang, rmarkdown,
        rstatix, rstudioapi, this.path, tidyr, writexl, xfun
Suggests: MASS, nnet, pbkrtest, testthat (>= 3.0.0), tibble
Config/testthat/edition: 3
SystemRequirements: Pandoc (>= 3.2)
NeedsCompilation: no
Packaged: 2026-05-30 18:29:52 UTC; sander
Author: Sander H. van Delden [aut, cre]
Maintainer: Sander H. van Delden <plantmind@proton.me>
URL: https://delde001.github.io/rfriend/
Config/roxygen2/version: 8.0.0
Repository: CRAN
Date/Publication: 2026-05-30 19:10:02 UTC
Built: R 4.5.2; ; 2026-05-30 22:39:46 UTC; unix
