# Generated by roxygen2: do not edit by hand

S3method(f_boxplot,data.frame)
S3method(f_boxplot,formula)
S3method(f_boxplot,integer)
S3method(f_boxplot,numeric)
S3method(f_outliers,data.frame)
S3method(f_outliers,formula)
S3method(f_outliers,integer)
S3method(f_outliers,numeric)
S3method(f_scan,data.frame)
S3method(f_scan,formula)
S3method(f_scan,integer)
S3method(f_scan,numeric)
S3method(f_stat_wizard,data.frame)
S3method(f_stat_wizard,formula)
S3method(f_summary,data.frame)
S3method(f_summary,formula)
S3method(f_t_test,default)
S3method(f_t_test,formula)
S3method(f_wilcox_test,default)
S3method(f_wilcox_test,formula)
S3method(plot,f_aov)
S3method(plot,f_bestNormalize)
S3method(plot,f_boxcox)
S3method(plot,f_friedman)
S3method(plot,f_glm)
S3method(plot,f_kruskal_test)
S3method(plot,f_lm)
S3method(plot,f_lmer)
S3method(plot,f_long)
S3method(plot,f_scan)
S3method(plot,f_t_test)
S3method(plot,f_wilcox_test)
S3method(predict,f_boxcox)
S3method(print,f_aov)
S3method(print,f_bestNormalize)
S3method(print,f_boxcox)
S3method(print,f_chisq_test)
S3method(print,f_friedman)
S3method(print,f_glm)
S3method(print,f_kruskal_test)
S3method(print,f_lm)
S3method(print,f_lmer)
S3method(print,f_model_comparison)
S3method(print,f_outliers)
S3method(print,f_scan)
S3method(print,f_stat_wizard)
S3method(print,f_summary)
S3method(print,f_t_test)
S3method(print,f_wilcox_test)
S3method(summary,f_long)
S3method(summary,f_scan)
export(df_to_table)
export(f_aov)
export(f_bestNormalize)
export(f_boxcox)
export(f_boxplot)
export(f_boxplot_worker)
export(f_chisq_test)
export(f_clear)
export(f_conditional_round)
export(f_corplot)
export(f_example_data)
export(f_factors)
export(f_friedman)
export(f_glm)
export(f_hist)
export(f_kruskal_test)
export(f_lm)
export(f_lmer)
export(f_load_packages)
export(f_long)
export(f_model_compare)
export(f_open_file)
export(f_outliers)
export(f_pander)
export(f_qqnorm)
export(f_remove_outliers)
export(f_rename_columns)
export(f_rename_vector)
export(f_scan)
export(f_setwd)
export(f_stat_wizard)
export(f_summary)
export(f_t_test)
export(f_theme)
export(f_wilcox_test)
import(ggplot2)
import(knitr)
import(lifecycle)
import(rmarkdown)
import(stats)
importFrom(DHARMa,plotResiduals)
importFrom(DHARMa,simulateResiduals)
importFrom(DHARMa,testDispersion)
importFrom(DHARMa,testResiduals)
importFrom(MuMIn,r.squaredGLMM)
importFrom(bestNormalize,bestNormalize)
importFrom(car,Anova)
importFrom(car,ncvTest)
importFrom(crayon,red)
importFrom(dplyr,select)
importFrom(emmeans,emmeans)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.off)
importFrom(grDevices,graphics.off)
importFrom(grDevices,hsv)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(grDevices,recordPlot)
importFrom(grDevices,replayPlot)
importFrom(grDevices,rgb)
importFrom(grDevices,rgb2hsv)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,stripchart)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(grid,grid.draw)
importFrom(grid,grid.newpage)
importFrom(grid,grid.raster)
importFrom(grid,unit)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(knitr,knit_child)
importFrom(knitr,knit_hooks)
importFrom(knitr,opts_knit)
importFrom(lme4,VarCorr)
importFrom(lme4,isSingular)
importFrom(lme4,lmerControl)
importFrom(lme4,ngrps)
importFrom(lme4,nobars)
importFrom(lme4,ranef)
importFrom(lmerTest,lmer)
importFrom(magick,image_read)
importFrom(magick,image_scale)
importFrom(nortest,ad.test)
importFrom(pander,pander)
importFrom(pander,panderOptions)
importFrom(rlang,.data)
importFrom(rstatix,dunn_test)
importFrom(rstatix,levene_test)
importFrom(rstudioapi,documentId)
importFrom(rstudioapi,documentPath)
importFrom(rstudioapi,getActiveDocumentContext)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,aov)
importFrom(stats,as.formula)
importFrom(stats,chisq.test)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,deviance)
importFrom(stats,df.residual)
importFrom(stats,dnorm)
importFrom(stats,drop1)
importFrom(stats,family)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,hatvalues)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,nobs)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,ppoints)
importFrom(stats,predict)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,shapiro.test)
importFrom(stats,sigma)
importFrom(stats,spline)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(this.path,this.dir)
importFrom(tidyr,all_of)
importFrom(tidyr,pivot_longer)
importFrom(utils,as.roman)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,tail)
importFrom(writexl,write_xlsx)
importFrom(xfun,dir_exists)
importFrom(xfun,split_lines)
