#' @import rmarkdown
#' @import knitr
#' @import ggplot2
#' @importFrom grDevices adjustcolor dev.off png rainbow rgb graphics.off  recordPlot replayPlot
#' @importFrom graphics abline arrows axis boxplot hist legend lines pairs par points polygon text stripchart strwidth layout mtext title
#' @importFrom stats aggregate aov as.formula cor kruskal.test drop1 median qchisq qqline qqnorm residuals sd shapiro.test spline chisq.test pnorm p.adjust AIC BIC anova coef deviance df.residual fitted formula gaussian glm hatvalues logLik model.frame nobs sigma terms update
#' @importFrom utils capture.output head tail combn
#' @importFrom multcompView multcompLetters
#' @importFrom pander pander panderOptions
#' @importFrom rstudioapi getActiveDocumentContext documentId documentPath
#' @importFrom stats ppoints qnorm quantile dnorm setNames family pchisq complete.cases predict
#' @importFrom rstatix levene_test dunn_test
#' @importFrom nortest ad.test
#' @importFrom writexl write_xlsx
#' @importFrom grid unit grid.newpage grid.draw grid.raster
#' @importFrom gridExtra grid.arrange arrangeGrob
#' @importFrom knitr knit_hooks opts_knit knit_child
#' @importFrom crayon red
#' @importFrom xfun split_lines dir_exists
#' @importFrom multcomp cld
#' @importFrom emmeans emmeans
#' @importFrom utils install.packages
#' @importFrom magick image_read image_scale
#' @importFrom bestNormalize bestNormalize
#' @importFrom stringr str_replace_all
#' @importFrom this.path this.dir
#' @importFrom MuMIn r.squaredGLMM
#' @importFrom dplyr select
#' @importFrom tidyr pivot_longer all_of
#' @importFrom magrittr %>%
#' @importFrom DHARMa plotResiduals testResiduals testDispersion simulateResiduals
#' @importFrom rlang .data
#' @importFrom lme4 ranef VarCorr ngrps isSingular lmerControl nobars
#' @importFrom lmerTest lmer
NULL
